from __future__ import unicode_literals

import creole

from .utils import binders_map, object_slug


class PinaxWikiHtmlEmitter(creole.HtmlEmitter):

    def __init__(self, wiki, root, link_rules=None):
        self.wiki = wiki
        super(PinaxWikiHtmlEmitter, self).__init__(root, link_rules)

    def link_emit(self, node):
        target = node.content
        if node.children:
            inside = self.emit_children(node)
        else:
            inside = self.html_escape(target)
        m = self.link_rules.addr_re.match(target)
        if m:
            if m.group("extern_addr"):
                return '<a href="{}">{}</a>'.format(self.attr_escape(target), inside)
            elif m.group("inter_wiki"):
                raise NotImplementedError
        slug = object_slug(self.wiki)
        page_url = binders_map()[slug].page_url(self.wiki, target)
        return '<a href="{}">{}</a>'.format(self.attr_escape(page_url), inside)


def creole_parse(wiki, text):
    document = creole.CreoleParser(text, blog_line_breaks=True).parse()
    return PinaxWikiHtmlEmitter(wiki, document).emit()



import os
import re
import uuid

from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.urls import reverse
from django.utils import timezone

from .conf import settings
from .utils import binders_map, object_slug

try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey


MEDIA_RE = re.compile(r"w/file-download/(\d+)/")


class Wiki(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.IntegerField()
    content_object = GenericForeignKey("content_type", "object_id")

    def latest_edits(self):
        return Revision.objects.filter(page__wiki=self).order_by("-created_at")

    class Meta:
        unique_together = [("content_type", "object_id")]


class Page(models.Model):
    wiki = models.ForeignKey(Wiki, related_name="pages", null=True, on_delete=models.CASCADE)
    slug = models.SlugField()

    def get_absolute_url(self):
        slug = object_slug(self.wiki)
        return binders_map()[slug].page_url(self.wiki, self.slug)

    def get_edit_url(self):
        slug = object_slug(self.wiki)
        return binders_map()[slug].edit_url(self.wiki, self.slug)

    class Meta:
        unique_together = [("wiki", "slug")]


class Revision(models.Model):
    page = models.ForeignKey(Page, related_name="revisions", on_delete=models.CASCADE)
    content = models.TextField(help_text="Use markdown to mark up your text")
    content_html = models.TextField()
    message = models.TextField(blank=True, help_text="Leave a helpful message about your change")
    created_ip = models.GenericIPAddressField()
    created_at = models.DateTimeField(default=timezone.now)
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="revisions_created", on_delete=models.CASCADE)
    media = models.ManyToManyField("MediaFile", blank=True, related_name="revisions")

    def parse(self):
        self.content_html = settings.PINAX_WIKI_PARSE(self.page.wiki, self.content)

    def process_media(self):
        pks = MEDIA_RE.findall(self.content)
        media = MediaFile.objects.filter(user=self.created_by, pk__in=pks)
        self.media.clear()
        for mf in media:
            self.media.add(mf)

    def save(self, *args, **kwargs):
        super(Revision, self).save(*args, **kwargs)
        self.process_media()

    class Meta:
        get_latest_by = "created_at"


def uuid_filename(instance, filename):
    ext = filename.split(".")[-1]
    filename = "{0}.{1}".format(uuid.uuid4(), ext)
    return os.path.join("revision-files", filename)


class MediaFile(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="media_files", on_delete=models.CASCADE)
    created = models.DateTimeField(editable=False, default=timezone.now)
    filename = models.CharField(max_length=255)
    file = models.FileField(upload_to=uuid_filename)

    def download_url(self):
        return reverse("pinax_wiki_file_download", args=[self.pk, os.path.basename(self.filename)])

    def __unicode__(self):
        return self.filename



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-27 17:00
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import pinax.wiki.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MediaFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('filename', models.CharField(max_length=255)),
                ('file', models.FileField(upload_to=pinax.wiki.models.uuid_filename)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='media_files', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField()),
            ],
        ),
        migrations.CreateModel(
            name='Revision',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField(help_text='Use markdown to mark up your text')),
                ('content_html', models.TextField()),
                ('message', models.TextField(blank=True, help_text='Leave a helpful message about your change')),
                ('created_ip', models.GenericIPAddressField()),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='revisions_created', to=settings.AUTH_USER_MODEL)),
                ('media', models.ManyToManyField(blank=True, related_name='revisions', to='pinax_wiki.MediaFile')),
                ('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='revisions', to='pinax_wiki.Page')),
            ],
            options={
                'get_latest_by': 'created_at',
            },
        ),
        migrations.CreateModel(
            name='Wiki',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.IntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
        ),
        migrations.AddField(
            model_name='page',
            name='wiki',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='pages', to='pinax_wiki.Wiki'),
        ),
        migrations.AlterUniqueTogether(
            name='wiki',
            unique_together=set([('content_type', 'object_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='page',
            unique_together=set([('wiki', 'slug')]),
        ),
    ]



import os

from django.conf.urls import url

from .conf import settings
from .views import edit, file_download, file_upload, index, page

app_name = "pinax_wiki"

urlpatterns = [
    url(r"^file-download/(\d+)/([^/]+)$", file_download, name="file_download"),
    url(r"^file-upload/$", file_upload, name="file_upload")
]

for binder in settings.PINAX_WIKI_BINDERS:
    urlpatterns += [
        url(os.path.join(binder.root, r"$"), index, {"binder": binder}, name=binder.index_url_name),
        url(os.path.join(binder.root, r"(?P<slug>[^/]+)/$"), page, {"binder": binder}, name=binder.page_url_name),
        url(os.path.join(binder.root, r"(?P<slug>[^/]+)/edit/$"), edit, {"binder": binder}, name=binder.edit_url_name),
    ]
